/*
 * Copyright (c) 2009-2011 Yoshikazu Kuramochi
 * All rights reserved.
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

package ch.kuramo.javie.app.project;

import ch.kuramo.javie.api.Size2i;
import ch.kuramo.javie.api.Vec3d;
import ch.kuramo.javie.app.InjectorHolder;
import ch.kuramo.javie.core.LayerComposition;
import ch.kuramo.javie.core.LayerNature;
import ch.kuramo.javie.core.TextLayer;
import ch.kuramo.javie.core.services.Font;
import ch.kuramo.javie.core.services.FontList;
import ch.kuramo.javie.core.services.GLGlobal;
import ch.kuramo.javie.core.services.ProjectElementFactory;

import com.google.inject.Inject;

public class NewTextLayerOperation extends AbstractNewLayerOperation<TextLayer> {

	@Inject
	private FontList _fontList;

	@Inject
	private GLGlobal _glGlobal;


	public NewTextLayerOperation(ProjectManager projectManager, LayerComposition comp) {
		super(projectManager, comp, "新規テキスト");
		InjectorHolder.getInjector().injectMembers(this);
	}

	@Override
	protected TextLayer createNewLayer(
			ProjectManager projectManager, LayerComposition comp, ProjectElementFactory factory) {

		TextLayer layer = factory.newTextLayer();
		layer.setName(projectManager.getUnusedLayerName(comp, "テキスト"));

		Font font = _fontList.defaultFont();
		if (font != null) {
			layer.getFont().reset(font.psName);
		}

		Size2i compSize = comp.getSize();
		Vec3d compCenter = new Vec3d(compSize.width/2d, compSize.height/2d);
		layer.getPosition().reset(compCenter);

		if (_glGlobal.getMaxSamples() > 0) {
			LayerNature.setCTCR(layer, true);
		}

		return layer;
	}

}
