/*
 * Copyright (c) 2009-2011 Yoshikazu Kuramochi
 * All rights reserved.
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

package ch.kuramo.javie.core;

import java.util.Collection;

import org.mozilla.javascript.Scriptable;

import ch.kuramo.javie.api.Color;
import ch.kuramo.javie.api.IAnimatableColor;
import ch.kuramo.javie.core.expression.ColorProperty;
import ch.kuramo.javie.core.expression.ExpressionUtils;

public class AnimatableColor extends ArithmeticalAnimatableValue<Color> implements IAnimatableColor {

	public AnimatableColor(Color staticValue, Collection<Keyframe<Color>> keyframes, String expression) {
		super(staticValue, keyframes, expression, Color.COLORLESS_TRANSPARENT, Color.WHITE);
	}

	public AnimatableColor(Color defaultValue) {
		super(defaultValue, Color.COLORLESS_TRANSPARENT, Color.WHITE);
	}

	@Override
	public Color clamp(Color value) {
		// AnimatableColorは、この時点でのクランプ処理をサポートしない。
		return value;
	}

	public Color jsToJava(Object jsValue) {
		Object o = ExpressionUtils.toDoubleOrDoubleArray(jsValue);

		if (o == null) {
			return Color.BLACK;
		}

		if (o instanceof double[]) {
			double[] array = (double[]) o;
			if (array.length < 4) {
				double[] tmp = array;
				array = new double[] { 0, 0, 0, 1 };
				for (int i = 0; i < tmp.length; ++i) {
					array[i] = tmp[i];
				}
			}
			return valueOf(array);
		}

		double d = (Double) o;
		return new Color(d, d, d, 1);
	}

	@Override
	protected double[] toArray(Color value) {
		return new double[] { value.rawRed, value.rawGreen, value.rawBlue, value.rawAlpha };
	}

	@Override
	protected Color valueOf(double[] d) {
		return new Color(d[0], d[1], d[2], d[3]);
	}

	public Scriptable createExpressionElement(final CoreContext context) {
		ColorProperty cp = new ColorProperty() {
			public Color getValue()				{ return value(context); }
			public Color valueAtTime(double t)	{ return AnimatableColor.this.valueAtTime(t, context); }
		};
		return context.toNativeJavaObject(cp, null);
	}

}
